/**
 * lerna-utils
 * Internal utilities for interacting with a lerna monorepo
 */
/// <reference types="node" />
import * as fs from 'fs';
import * as t from 'io-ts';
import * as E from 'fp-ts/Either';
import * as F from 'fluture';
import * as most from 'most';
import { LernaPackage } from '@typescript-tools/io-ts/dist/lib/LernaPackage';
export declare type DependencyGraph = {
    [packageName: string]: LernaPackage[];
};
export declare const readFile: (file: fs.PathLike) => F.FutureInstance<NodeJS.ErrnoException, string>;
export declare const writeFile: (file: fs.PathLike) => (contents: string) => F.FutureInstance<NodeJS.ErrnoException, void>;
export declare const fromFuture: <L extends Error, R>(future: F.FutureInstance<L, R>) => most.Stream<R>;
export declare function trace(logger: typeof console.log, ...tag: any[]): <T>(value: T) => T;
export declare function withEncode<C extends t.Any, O>(codec: C, encode: (...a: Parameters<C['encode']>) => O, name?: string): t.Type<C['_A'], O, C['_I']>;
export declare function validationErrors(typeAlias: string, errors: t.Errors): string;
export declare function decodeCommandLineArguments<C extends t.Mixed>(codec: C, docstring: string, { input }?: {
    input?: string[];
}): E.Either<t.Errors, C['_O']>;
/**
 * Get list of all lerna packages from `lerna list --all`.
 */
export declare function lernaPackages(root: string): F.FutureInstance<unknown, LernaPackage[]>;
export declare function dependencyGraph(root: string): F.FutureInstance<unknown, DependencyGraph>;
//# sourceMappingURL=lerna-utils.d.ts.map