"use strict";
/**
 * lerna-utils
 * Internal utilities for interacting with a lerna monorepo
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dependencyGraph = exports.lernaPackages = exports.decodeCommandLineArguments = exports.validationErrors = exports.withEncode = exports.trace = exports.fromFuture = exports.writeFile = exports.readFile = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const t = __importStar(require("io-ts"));
const A = __importStar(require("fp-ts/Array"));
const E = __importStar(require("fp-ts/Either"));
const O = __importStar(require("fp-ts/Option"));
const NEA = __importStar(require("fp-ts/NonEmptyArray"));
const F = __importStar(require("fluture"));
const most = __importStar(require("most"));
const debug_1 = __importDefault(require("debug"));
const execa_1 = __importDefault(require("execa"));
const most_buffer_1 = __importDefault(require("most-buffer"));
const pipeable_1 = require("fp-ts/pipeable");
const function_1 = require("fp-ts/lib/function");
const shades_1 = require("shades");
const LernaPackage_1 = require("@typescript-tools/io-ts/dist/lib/LernaPackage");
const StringifiedJSON_1 = require("@typescript-tools/io-ts/dist/lib/StringifiedJSON");
const docopt_1 = require("docopt");
const debug = {
    options: debug_1.default('options')
};
exports.readFile = (file) => F.node(done => fs.readFile(file, 'utf8', done));
exports.writeFile = (file) => (contents) => F.node(done => fs.writeFile(file, contents, done));
exports.fromFuture = (future) => most.fromPromise(F.promise(future));
/* eslint-disable @typescript-eslint/no-explicit-any */
function trace(logger, ...tag) {
    return function trace(value) {
        if (tag.length > 0) {
            logger(...tag, value);
        }
        else {
            logger(value);
        }
        return value;
    };
}
exports.trace = trace;
// TODO: pull this into io-ts-types
function withEncode(codec, encode, name = codec.name) {
    function clone(t) {
        const r = Object.create(Object.getPrototypeOf(t));
        Object.assign(r, t);
        return r;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const r = clone(codec);
    r.encode = encode;
    r.name = name;
    return r;
}
exports.withEncode = withEncode;
function validationErrors(typeAlias, errors) {
    /**
     * Inspired by
     * https://github.com/mmkal/ts/blob/94a9ba8f2931c9c91122d00b0bf1bd21b2be05cd/packages/io-ts-extra/src/reporters.ts#L11.
     */
    return errors.map((error) => {
        var _a;
        const name = typeAlias || ((_a = error.context[0]) === null || _a === void 0 ? void 0 : _a.type.name);
        const lastType = error.context.length && error.context[error.context.length - 1].type.name;
        const path = name + error.context.map((c) => c.key).join('.');
        return `Invalid value '${JSON.stringify(error.value)}' supplied to ${path}, expected ${lastType}.`;
    }).join('\n');
}
exports.validationErrors = validationErrors;
function decodeCommandLineArguments(codec, docstring, { input = process.argv.slice(2) } = {}) {
    return pipeable_1.pipe(input, argv => docopt_1.docopt(docstring, { argv, help: true, exit: true }), codec.decode.bind(null), E.map(codec.encode.bind(null)), E.map(trace(debug.options, 'Arguments')));
}
exports.decodeCommandLineArguments = decodeCommandLineArguments;
/**
 * Get list of all lerna packages from `lerna list --all`.
 */
function lernaPackages(root) {
    return F.Future((reject, resolve) => {
        const subcommand = execa_1.default.command('npx lerna list --all --json', { cwd: root });
        subcommand
            .then(({ stdout }) => pipeable_1.pipe(stdout, StringifiedJSON_1.StringifiedJSON(t.array(LernaPackage_1.LernaPackage)).decode.bind(null), E.fold(reject, resolve)))
            .catch(reject);
        return function onCancel() {
            subcommand.cancel();
        };
    });
}
exports.lernaPackages = lernaPackages;
function dependencyGraph(root) {
    const Dependencies = t.partial({
        dependencies: t.record(t.string, t.string),
        devDependencies: t.record(t.string, t.string),
        peerDependencies: t.record(t.string, t.string)
    });
    return lernaPackages(root)
        .pipe(F.chain(packages => F.attemptP(() => __awaiter(this, void 0, void 0, function* () {
        return most.from(packages)
            // parse each package's package.json file
            .chain(pkg => most.of(pkg.location)
            .map(dir => path.resolve(dir, 'package.json'))
            .map(exports.readFile)
            .chain(exports.fromFuture)
            .map(contents => pipeable_1.pipe(E.parseJSON(contents, E.toError), E.mapLeft(error => `Unable to parse ${pkg.location}/package.json, ${error.message}`), E.map(contents => ({ pkg, contents })))))
            .thru(most_buffer_1.default())
            // report all parse errors, if any
            .chain(results => pipeable_1.pipe(NEA.fromArray(A.lefts(results)), O.fold(() => most.of(A.rights(results)), 
        // Nasty type assertion, but at least it'll complain if we change the type of the above line
        failures => most.throwError(new Error([`Aborting, errors encountered:`].concat(failures).join('\n'))))))
            // map package's package.json file into a list of internal dependencies
            .map(packages => {
            // map of a package name to its metadata
            const internalPackages = packages
                .map(shades_1.get('pkg'))
                .reduce((acc, pkg) => Object.assign(acc, { [pkg.name]: pkg }), {});
            // map of a package name to its internal dependencies
            const internalDependencies = packages
                .reduce((acc, { pkg, contents }) => Object.assign(acc, {
                [pkg.name]: pipeable_1.pipe(Dependencies.decode(contents), E.map(contents => {
                    var _a, _b, _c;
                    return [
                        ...Object.keys((_a = contents.dependencies) !== null && _a !== void 0 ? _a : {}),
                        ...Object.keys((_b = contents.devDependencies) !== null && _b !== void 0 ? _b : {}),
                        ...Object.keys((_c = contents.peerDependencies) !== null && _c !== void 0 ? _c : {}),
                    ];
                }), E.map(A.filter(dependency => internalPackages.hasOwnProperty(dependency))), E.getOrElse(function_1.constant([])), A.map(dependency => internalPackages[dependency]))
            }), {});
            // Recursively list all internal dependencies, even transitive ones
            const allInternalDependencies = (pkg) => {
                const processed = new Set();
                const deps = [];
                let next = internalDependencies[pkg];
                do {
                    next.forEach(dependency => {
                        processed.add(dependency.name);
                        deps.push(dependency);
                    });
                    next = pipeable_1.pipe(next, A.map(dependency => { var _a; return (_a = internalDependencies[dependency.name]) !== null && _a !== void 0 ? _a : []; }), A.flatten, A.filter(dependency => !processed.has(dependency.name)));
                } while (!A.isEmpty(next));
                return deps;
            };
            return packages.reduce((acc, { pkg }) => Object.assign(acc, { [pkg.name]: allInternalDependencies(pkg.name) }), {});
        })
            // stream only emits a single value
            .reduce((_, value) => value, {});
    }))));
}
exports.dependencyGraph = dependencyGraph;
//# sourceMappingURL=lerna-utils.js.map